@@ -19,42 +19,41 @@ from pywss.closing import Closing
 from pywss.routing import Route
 from pywss.openapi import openapi_ui_template, merge_dict, parameters_filter
 
-__version__ = '0.1.6'
+__version__ = '0.1.9'
 
 
 class Context:
     _handler_index = 0
     _flush_header = False
 
-    def __init__(self, app, fd, rfd, method, path, paths, version, headers, route, handlers, address):
-        self.app = app
-        self.fd = fd
-        self.rfd = rfd
-        self.method = method
-        self.version = version
-        self.headers = headers
-        self.cookies = parse_cookies(headers)
-        self.path = path
-        self.paths = paths
-        self.params = parse_params(path)
-        self.route = route
-        self._handlers = handlers
-        self.address = address
-        self.log: loggus.Entry = None
+    def __init__(self, app, fd, address, log, rfd, method, path, paths, version, headers, route, handlers):
+        self.app: 'App' = app
+        self.log: loggus.Entry = log
+        self.fd: socket.socket = fd
+        self.rfd: BufferedReader = rfd
+        self.method: str = method
+        self.version: str = version
+        self.headers: dict = headers
+        self.cookies: dict = parse_cookies(headers)
+        self.path: str = path
+        self.paths: dict = paths
+        self.params: dict = parse_params(path)
+        self.route: str = route
+        self._handlers: list = handlers
+        self.address: tuple = address
+        self.data: data = data()  # data save for user
 
-        self.content_length = int(headers.get("Content-Length", 0))
-        self.content = b""
-        if self.content_length:
-            self.content = rfd.read(self.content_length)
+        self.content_length: int = int(headers.get("Content-Length", 0))
+        self.content: bytes = b""  # default empty, use self.body() to instead
 
-        self.response_status_code = 200
-        self.response_headers = {
+        self.response_status_code: int = 200
+        self.response_headers: dict = {
             "Server": "Pywss",
             "PywssVersion": __version__,
         }
-        self.response_body = []
+        self.response_body: list = []
 
-    def next(self):
+    def next(self) -> None:
         if self._handler_index >= len(self._handlers):
             return
         index = self._handler_index
@@ -62,56 +61,59 @@ class Context:
         self._handlers[index](self)
 
     def json(self):
+        return json.loads(self.body().decode())  # not check Content-Type: application/json
+
+    def form(self) -> dict:
         resp = {}
-        try:
-            ct = self.headers.get("Content-Type").strip()
-            if not ct.startswith("application/json"):
-                return resp
-            resp = json.loads(self.content.decode())
-        except:
-            pass
+        ct = self.headers.get("Content-Type").strip()
+
+        if ct == "application/x-www-form-urlencoded":
+            for value in self.body().decode().strip().split("&"):
+                k, v = value.split("=", 1)
+                resp[unquote(k)] = unquote(v)
+            return resp
+        elif not ct.startswith("multipart/form-data"):
+            raise Exception(f"not support form Content-Type:{ct}")
+
+        # parse form-data boundary
+        boundary = ""
+        for v in ct.split(";"):
+            v = v.strip()
+            if v.startswith("boundary="):
+                boundary = v.replace("boundary=", "")
+                break
+        if not boundary:
+            raise Exception(f"invalid form-data, with not boundary")
+        # parse form-data
+        for data in self.body().decode().split(f"--{boundary}"):
+            data = data.strip()
+            if not data.startswith("Content-Disposition"):
+                continue
+            h, v = data.split("\r\n\r\n", 1)
+            name = re.search("name=\"(.*?)\"", h).group(1)
+            resp[name] = v
         return resp
 
-    def form(self):
-        resp = {}
-        try:
-            ct = self.headers.get("Content-Type").strip()
-            if not ct.startswith("multipart/form-data"):
-                return resp
-            boundary = ""
-            for v in ct.split(";"):
-                v = v.strip()
-                if v.startswith("boundary="):
-                    boundary = v.replace("boundary=", "")
-            if not boundary:
-                return resp
-            for data in self.content.decode().split(f"--{boundary}"):
-                data = data.strip()
-                if not data.startswith("Content-Disposition"):
-                    continue
-                h, v = data.split("\r\n\r\n", 1)
-                name = re.search("name=\"(.*?)\"", h).group(1)
-                resp[name] = v
-        except:
-            pass
-        return resp
+    def body(self) -> bytes:
+        if not self.content and self.content_length:
+            self.content = self.rfd.read(self.content_length)
+        return self.content
 
-    def body(self):
-        return self.content.decode()
-
-    def set_header(self, k, v):
+    def set_header(self, k, v) -> None:
         header = []
         for key in k.split("-"):
             header.append(key[0].upper() + key[1:].lower())
         self.response_headers["-".join(header)] = v
 
-    def set_content_length(self, size: int):
+    def set_content_length(self, size: int) -> None:
         self.response_headers["Content-Length"] = self.response_headers.get("Content-Length", 0) + size
 
-    def set_content_type(self, v):
+    def set_content_type(self, v) -> None:
         self.response_headers.setdefault("Content-Type", v)
 
-    def set_cookie(self, key, value, maxAge=None, expires=None, path="/", domain=None, secure=False, httpOnly=False):
+    def set_cookie(
+            self, key, value, maxAge=None, expires=None, path="/", domain=None, secure=False, httpOnly=False
+    ) -> None:
         buf = [f"{key}={value}"]
         if isinstance(maxAge, timedelta):
             maxAge = (maxAge.days * 60 * 60 * 24) + maxAge.seconds
@@ -163,14 +165,14 @@ class Context:
             buf.append(f"{k}={v}")
         self.response_headers["Set-Cookie"] = "; ".join(buf)
 
-    def set_status_code(self, status_code):
+    def set_status_code(self, status_code) -> None:
         self.response_status_code = status_code
 
-    def redirect(self, url, status_code=StatusFound):
+    def redirect(self, url, status_code=StatusFound) -> None:
         self.set_status_code(status_code)
         self.set_header("Location", url)
 
-    def write(self, body):
+    def write(self, body) -> None:
         if isinstance(body, (str, bytes)):
             self.write_text(body)
         elif isinstance(body, (dict, list)):
@@ -178,20 +180,20 @@ class Context:
         elif isinstance(body, BufferedReader):
             self.write_file(body)
 
-    def write_text(self, data: str):
+    def write_text(self, data: str) -> None:
         if isinstance(data, str):
             data = data.encode()
         self.set_content_length(len(data))
         self.set_content_type("text/html; charset=utf-8")
         self.response_body.append(data)
 
-    def write_json(self, data):
+    def write_json(self, data) -> None:
         data = json.dumps(data, ensure_ascii=False).encode()
         self.set_content_length(len(data))
         self.set_content_type("application/json")
         self.response_body.append(data)
 
-    def write_file(self, file):
+    def write_file(self, file) -> None:
         if isinstance(file, str) and os.path.exists(file):
             file = open(file, "rb")
         if not isinstance(file, BufferedReader):
@@ -199,13 +201,13 @@ class Context:
         self.set_content_length(os.stat(file.fileno())[6])
         self.response_body.append(file)
 
-    def ws_read(self):  # by WebSocketUpgrade
+    def ws_read(self) -> bytes:  # impl by WebSocketUpgrade
         raise NotImplementedError
 
-    def ws_write(self, body):  # by WebSocketUpgrade
+    def ws_write(self, body) -> None:  # impl by WebSocketUpgrade
         raise NotImplementedError
 
-    def flush(self):
+    def flush(self) -> None:
         if not self._flush_header:
             data = [f"{self.version} {self.response_status_code} Pywss"]
             for k, v in self.response_headers.items():
@@ -230,7 +232,6 @@ class Context:
 class App:
 
     def __init__(self, base_route="", base_handlers=None):
-        self.running = True
         self.base_route = f"/{base_route.strip().strip('/')}" if base_route else base_route
         self.base_handlers = list(base_handlers) if base_handlers else []
         self.head_match_routes = []
@@ -240,6 +241,7 @@ class App:
         self.openapi_data = {
             "paths": defaultdict(dict),
         }
+        self.running = False
         Closing.add_close(self.close)
 
     def register(self, method, route, handlers) -> None:
@@ -307,6 +309,7 @@ class App:
                     "handlers": [handler.__name__ for handler in v],
                 }).info(f"bind route")
         self.full_match_routes = routes
+        self.running = True
 
     def party(self, route, *handlers) -> 'App':
         route = f"{self.base_route}/{route.strip().strip('/')}"
@@ -381,53 +384,60 @@ class App:
             openapi_ui_css_url,
         )))
 
-    def _(self, request, address) -> None:
+    def _(self, request: socket.socket, address: tuple) -> None:
         log = self.log
         try:
             rfd = request.makefile("rb", -1)
-            method, path, version, err = parse_request_line(rfd)
-            if err:
-                request.sendall(b"HTTP/1.1 400 BadRequest\r\n")
-                return
-            log = log.update(method=method, path=path)
-            hes, err = parse_headers(rfd)
-            if err:
-                request.sendall(b"HTTP/1.1 400 BadRequest\r\n")
-                log.error(err)
-                return
-            # full match
-            paths = {}
-            route = f"{method.upper()}/{path.split('?', 1)[0].strip('/')}"
-            handlers = self.full_match_routes.get(route, None)
-            # parser match
-            if not handlers:
-                for r, v in self.parse_match_routes:
-                    fix, ps = r.match(route)
-                    if fix:
-                        route = r.route
-                        handlers = v
-                        paths = ps
-                        break
-            # head match
-            if not handlers:
-                for r, v in self.head_match_routes:
-                    if route.startswith(r):
-                        route = r
-                        handlers = v
-                        break
-            if not handlers:
-                request.sendall(b"HTTP/1.1 404 NotFound\r\n")
-                log.warning("No Handler")
-                return
-            ctx = Context(self, request, rfd, method, path, paths, version, hes, route, handlers, address)
-            ctx.log = log
-            ctx.next()
-            ctx.flush()
+            while True:
+                method, path, version, err = parse_request_line(rfd)
+                if err:
+                    request.sendall(b"HTTP/1.1 400 BadRequest\r\n")
+                    return
+                log = log.update(method=method, path=path)
+                hes, err = parse_headers(rfd)
+                if err:
+                    request.sendall(b"HTTP/1.1 400 BadRequest\r\n")
+                    log.error(err)
+                    return
+                # check keep alive
+                if hes.get("Connection", "").lower() == "close":
+                    break
+                # full match
+                paths = {}
+                route = f"{method.upper()}/{path.split('?', 1)[0].strip('/')}"
+                handlers = self.full_match_routes.get(route, None)
+                # parser match
+                if not handlers:
+                    for r, v in self.parse_match_routes:
+                        fix, ps = r.match(route)
+                        if fix:
+                            route = r.route
+                            handlers = v
+                            paths = ps
+                            break
+                # head match
+                if not handlers:
+                    for r, v in self.head_match_routes:
+                        if route.startswith(r):
+                            route = r
+                            handlers = v
+                            break
+                if not handlers:
+                    request.sendall(b"HTTP/1.1 404 NotFound\r\n")
+                    log.warning("No Handler")
+                    return
+                ctx = Context(self, request, address, log, rfd, method, path, paths, version, hes, route, handlers)
+                ctx.next()
+                ctx.flush()
         except ConnectionAbortedError:
             log.error("connect abort")
         except:
             log.traceback()
         finally:
+            try:
+                request.shutdown(socket.SHUT_WR)
+            except OSError:
+                pass
             request.close()
 
     def close(self) -> None:
@@ -452,7 +462,17 @@ class App:
                         continue
                     request, address = sock.accept()
                     threading.Thread(target=self._, args=(request, address)).start()
-        for i in range(int(grace) + 1):
+        for i in range(grace):
             self.log.update(hit=i + 1, grace=grace).warning("server closing")
             time.sleep(1)
-        self.log.panic("server closed")
+        self.log.warning("server closed")
+        sock.close()
+
+
+class data(dict):
+
+    def __setattr__(self, key, value):
+        self[key] = value
+
+    def __getattr__(self, item):
+        return self.get(item, None)
